//This code used the onboard DAC to sample the signal and the take the FFT of it
#define _USE_MATH_DEFINES
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <Scheduler.h>

#define PI	M_PI	/* pi to machine precision, defined in math.h */
#define TWOPI	(2.0*PI)
#define C       299792458
#define T       (.01475/2)
#define BW      334400000



void setup() {
       Serial.begin(115200);	
       pinMode(0,  INPUT);
       pinMode(13, INPUT);
       analogWriteResolution(10);
       analogReadResolution(14);
       analogWrite(DAC0,120);
}

void loop() {
  
        int i;
	int Nx;
	int NFFT;
	double *x;
	double *X;
        double *mag;

	
	Nx = 2000;
	printf("Nx = %d\n", Nx);
	x = (double *) malloc(Nx * sizeof(double));
        //Sampelint pin zero
        if(digitalRead(13)==HIGH){
            while(digitalRead(13)==HIGH){}
            while(digitalRead(13)==LOW){}
        }
        if(digitalRead(13)==LOW){
            //while(digitalRead(0)==HIGH){}
            while(digitalRead(13)==LOW){}
        }
        float tic = micros();
	for(i=0; i<Nx; i++)
	{
		x[i] = analogRead(0);
                delayMicroseconds(10);
	}
        float toc = micros();
        float time=(toc-tic)*pow(10,-6);
        int sampf=Nx/time;
        
        //Printing results
        Serial.print("time =");
        Serial.println(time);
        Serial.print("Sampeling frequincy =");
        Serial.println(sampf);
        //Normalization
        double max_val=x[0];
        double low_val=x[0];
        for(int i=0;i<Nx;i++)
        {
            if(x[i]>max_val){
              max_val=x[i];
            }
            if(x[i]<low_val)
            {
               low_val=x[i];
            }
        }
        double delta=max_val-low_val;
        for(int i=0;i<Nx;i++)
        {
            x[i]=(x[i]-low_val-delta/2)/(delta/2);
        }
        Serial.println(max_val);
        Serial.println(low_val);
        
        /* calculate NFFT as the next higher power of 2 >= Nx */
	NFFT = (int)pow(2.0, ceil(log((double)Nx)/log(2.0)));
	printf("NFFT = %d\n", NFFT);

	/* allocate memory for NFFT complex numbers (note the +1) */
	X = (double *) malloc((2*NFFT+1) * sizeof(double));
        mag = (double *) malloc((NFFT+1) * sizeof(double));
	/* Storing x(n) in a complex array to make it work with four1. 
	This is needed even though x(n) is purely real in this case. */
	for(i=0; i<Nx; i++)
	{
		X[2*i+1] = x[i];
		X[2*i+2] = 0.0;
	}
	/* pad the remainder of the array with zeros (0 + 0 j) */
	for(i=Nx; i<NFFT; i++)
	{
		X[2*i+1] = 0.0;
		X[2*i+2] = 0.0;
	}
	/* calculate FFT */
	four1(X, NFFT, 1);
        for(i=0; i<NFFT; i++)
	{
  	    mag[i]=sqrt(pow(X[2*i+1],2)+pow(X[2*i+2],2));	

	}
        //Location of the maximum
        double mag_max=mag[0];
        int index;
        for(i=0; i<NFFT/2; i++)
	{
  	    if(mag[i]>mag_max)
            {
                mag_max=mag[i];
                index=i;
            }	

	}
        //Print out the results
        float frequincy=(index*PI/(NFFT/2))*(sampf/TWOPI);
        Serial.print("Frequincy of the sampled signal =");
        Serial.println(frequincy);
        float distance = frequincy*C*T/(2*BW);
        float inch=distance*39.37;
        Serial.print("DISTANCE =");
        Serial.println(distance);
        Serial.print("DISTANCE IN INCHES =");
        Serial.println(inch);
        Serial.print("\n\n\n");
        delay(1000);
////	printf("\nFFT:\n");
////	for(i=0; i<NFFT; i++)
////	{
////        Serial.print(i);
////        Serial.print("  ");
////        Serial.println(mag[i]);
//////        Serial.print(" ");
//////        Serial.print(X[2*i+1]);
//////        Serial.print(" ");
//////        Serial.println(X[2*i+2]);		
////  //printf("X[%d] = (%.2f + j %.2f)\n", i, X[2*i+1], X[2*i+2]);
////	}
//
////	/* calculate IFFT */
////	four1(X, NFFT, -1);
////
////	/* normalize the IFFT */
////	for(i=0; i<NFFT; i++)
////	{
////		X[2*i+1] /= NFFT;
////		X[2*i+2] /= NFFT;
////	}
////
////	printf("\nComplex sequence reconstructed by IFFT:\n");
////	for(i=0; i<NFFT; i++)
////	{
////		printf("x[%d] = (%.2f + j %.2f)\n", i, X[2*i+1], X[2*i+2]);
////	}
        for(i=0; i<NFFT; i++)
	{
  	    mag[i]=0;	

	}
        free(X);
        free(x);
        free(mag);
}

//Function used to claculate the FFT
void four1(double data[], int nn, int isign)
{
    int n, mmax, m, j, istep, i;
    double wtemp, wr, wpr, wpi, wi, theta;
    double tempr, tempi;
    
    n = nn << 1;
    j = 1;
    for (i = 1; i < n; i += 2) {
	if (j > i) {
	    tempr = data[j];     data[j] = data[i];     data[i] = tempr;
	    tempr = data[j+1]; data[j+1] = data[i+1]; data[i+1] = tempr;
	}
	m = n >> 1;
	while (m >= 2 && j > m) {
	    j -= m;
	    m >>= 1;
	}
	j += m;
    }
    mmax = 2;
    while (n > mmax) {
	istep = 2*mmax;
	theta = TWOPI/(isign*mmax);
	wtemp = sin(0.5*theta);
	wpr = -2.0*wtemp*wtemp;
	wpi = sin(theta);
	wr = 1.0;
	wi = 0.0;
	for (m = 1; m < mmax; m += 2) {
	    for (i = m; i <= n; i += istep) {
		j =i + mmax;
		tempr = wr*data[j]   - wi*data[j+1];
		tempi = wr*data[j+1] + wi*data[j];
		data[j]   = data[i]   - tempr;
		data[j+1] = data[i+1] - tempi;
		data[i] += tempr;
		data[i+1] += tempi;
	    }
	    wr = (wtemp = wr)*wpr - wi*wpi + wr;
	    wi = wi*wpr + wtemp*wpi + wi;
	}
	mmax = istep;
    }
}
